@@ -43,16 +43,21 @@ def get_file_experimental_breath_meta(file, tve_pos=True, ignore_missing_bes=Tru
 
 def _get_file_breath_meta(func, file, tve_pos, ignore_missing_bes, rel_bn_interval, vent_bn_interval, to_data_frame, spec_vent_bns, spec_rel_bns):
     if isinstance(file, str):
-        file = open(file)
+        file = open(file, encoding='ascii', errors='ignore')
     if "experimental" in func.__name__:
         array = [EXPERIMENTAL_META_HEADER]
     else:
         array = [META_HEADER]
-    # XXX add logic for accepting a raw_utils array
-    for breath in extract_raw(file, ignore_missing_bes,
-        rel_bn_interval=rel_bn_interval, vent_bn_interval=vent_bn_interval,
-        spec_vent_bns=spec_vent_bns, spec_rel_bns=spec_rel_bns):
-        array.append(func(breath))
+
+    # case that the file is just a raw_utils array of breaths
+    if isinstance(file, list):
+        for b in file:
+            array.append(func(b))
+    else:  # case the file is a file descriptor
+        for breath in extract_raw(file, ignore_missing_bes,
+            rel_bn_interval=rel_bn_interval, vent_bn_interval=vent_bn_interval,
+            spec_vent_bns=spec_vent_bns, spec_rel_bns=spec_rel_bns):
+            array.append(func(breath))
 
     if not to_data_frame:
         return array
@@ -168,9 +173,18 @@ def get_production_breath_meta(breath, tve_pos=True, calc_tv3=False, to_series=F
         epAUC = 0
     else:
         epAUC = float(simps(ePressure, dx=dt))
-    maxP = max(wPressure) #max pressure for whole breath
-    maxF = max(wFlow)
-    minF = min(wFlow)
+    try:
+        maxP = max(wPressure) #max pressure for whole breath
+    except ValueError:
+        maxP = np.nan
+    try:
+        maxF = max(wFlow)
+    except ValueError:
+        maxF = np.nan
+    try:
+        minF = min(wFlow)
+    except ValueError:
+        minF = np.nan
 
 
     # TODO: will probabily move this to somewhere else
@@ -296,6 +310,7 @@ def get_experimental_breath_meta(breath, tve_pos=True):
     pressure_itime_pip5 = SAM.calc_pressure_itime_by_pip(rel_time_array, pressure, PIP, 5)
     pressure_itime_pip6 = SAM.calc_pressure_itime_by_pip(rel_time_array, pressure, PIP, 6)
     pressure_itime_from_front = SAM.calc_pressure_itime_from_front(rel_time_array, pressure, PIP, peep, .4)
+    shear_p_itime = SAM.shear_transform(pressure, flow, dt) * dt
 
     # The array indices go like this
     #
@@ -310,11 +325,13 @@ def get_experimental_breath_meta(breath, tve_pos=True):
     # 39: dyn_compliance, 40: vol_at_05, 41: vol_at_076, 42: vol_at_1,
     # 43: pressure_itime4, 44: pressure_itime5, 45: pressure_itime6,
     # 46: pressure_itime_pip5, 47: pressure_itime_by_pip6, 48: pressure_itime_from_front
+    # 49: pressure_itime_shear,
     return non_experimental + [
         pef_to_zero, pef_plus_16_to_zero,
         mean_flow_from_pef, dyn_compliance, vol_at_05,
         vol_at_076, vol_at_1, pressure_itime4, pressure_itime5, pressure_itime6,
         pressure_itime_pip5, pressure_itime_pip6, pressure_itime_from_front,
+        shear_p_itime,
     ]
 
 
